(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "gigantes"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.html = exports.loadDomSelector = void 0;
    const gigantes_1 = require("gigantes");
    exports.loadDomSelector = (path) => {
        const module = require(path);
        const domSelectors = [];
        Object.keys(module).forEach((key) => {
            if (["SingleDomSelector", "MultipleDomSelector"].includes(module[key].constructor.name)) {
                delete require.cache[path];
                domSelectors.push({
                    name: key,
                    domSelector: grantType(module[key]),
                });
            }
        });
        return domSelectors;
    };
    const grantType = (domSelector) => {
        const source = {
            type: domSelector.constructor.name,
        };
        if (domSelector.prevElementSpec) {
            source.prevElementSpec = grantType(domSelector.prevElementSpec);
        }
        if (domSelector.parentElementSpec) {
            source.parentElementSpec = grantType(domSelector.parentElementSpec);
        }
        return Object.assign(domSelector, source);
    };
    const getDomNode = (domSelectors, body) => {
        let domNodes = [body];
        domSelectors.forEach((domSelector) => {
            domNodes = gigantes_1.nestMap(domNodes, (domNode) => {
                const domNodes = domSelector.getDomNode()(domNode);
                if (Array.isArray(domNodes)) {
                    return domNodes;
                }
                else {
                    return [domNodes];
                }
            });
        });
        return domNodes;
    };
    exports.html = (url, filePaths, domSelectorNames) => {
        try {
            const $ = gigantes_1.fetchSync(url);
            const body = new gigantes_1.DomNode($("body"), $);
            $("head").append(`<script>      
      function genMark() {
        $(".gigantes-mark").remove();
        $(".gigantes-focus").each((i, e) => {
          const height = $(e).css("height");
          const width = $(e).css("width");
          const offset = $(e).offset();
  
          const name = $(e)
            .attr("class")
            .split(" ")
            .find((c) => c.startsWith("name-"))
            .match(/name-(.*)/)[1];

          const $div = $("<div></div>");
          $div.text(name);
          $div.addClass("gigantes-mark");
          $div.css("height", height);
          $div.css("width", width);
          $div.css("position", "absolute");
          $div.css("background-color", "#ff0099aa");
          $div.css("border", "solid 4px #ff0099");
          $div.css("font-weight", "bold");
          $div.css("z-index", "10000");
          $div.css("color", "white");
          $div.css("padding", "4px");
          $div.offset(offset);
  
          $("body").append($div);
        });
      }

      var $element = $("body");
      var lastHeight = $("body").css('height');
      function checkForChanges()
      {
          if ($element.css('height') != lastHeight)
          {
              genMark()
              lastHeight = $element.css('height'); 
          }
      
          setTimeout(checkForChanges, 500);
      }

      $(function(){
        checkForChanges();
      });
      </script>`);
            const params = [];
            filePaths.forEach((filePath, i) => {
                params.push({
                    filePath,
                    name: domSelectorNames[i],
                });
            });
            const domSelectors = params.map(({ filePath, name }) => {
                return exports.loadDomSelector(filePath).find((domSelector) => domSelector.name === name).domSelector;
            });
            const name = domSelectorNames.reverse()[0];
            const domNodes = getDomNode(domSelectors, body);
            // console.log(
            //   JSON.stringify(
            //     nestMap(domNodes, (d: DomNode) => ({
            //       tagName: d.tagName,
            //       classNames: d.classNames,
            //     })),
            //     null,
            //     2
            //   )
            // );
            gigantes_1.nestMap(domNodes, (domNode) => {
                domNode.cheerio.addClass("gigantes-focus");
                domNode.cheerio.addClass(`name-${name}`);
                //   domNode.cheerio.append(`<span class="domselector-name">${name}</span>`);
            });
            return $.html();
        }
        catch (e) {
            console.error(e);
            return "";
        }
    };
});
